import {DOCUMENT} from "@angular/common";
import {Component, Inject, Renderer2} from "@angular/core";
import {Router, RouterStateSnapshot} from "@angular/router";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {LanguagesEnum} from "@app/shared/enums/languages.enum";
import {RoutesEnum, urlSeparator} from "@app/shared/enums/routes.enum";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {ChangeAppLanguage, ChangeAppTheme, InitApplication, Logout} from "@app/stores/app.action";
import {TranslateService} from "@ngx-translate/core";
import {Navigate} from "@ngxs/router-plugin";
import {Select, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {OAuth2Service} from "solidify-frontend";
import {environment} from "../environments/environment";
import {LocalStateEnum} from "./shared/enums/local-state.enum";

@Component({
  selector: "dlcm-root",
  templateUrl: "./app.component.html",
  styleUrls: ["./app.component.scss"],
})
export class AppComponent extends SharedAbstractPresentational {

  private static readonly _themeAttributeName: string = "theme";
  currentModule: string;
  logo: string;

  @Select((state: LocalStateModel) => state[LocalStateEnum.router].state) urlStateObs: Observable<RouterStateSnapshot>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.application].appLanguage) appLanguageObs: Observable<LanguagesEnum>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.application].isLoggedIn) isLoggedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.application].isApplicationInitialized) isApplicationInitializedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state[LocalStateEnum.application].theme) themeObs: Observable<ThemeEnum>;

  constructor(private store: Store,
              private oauthService: OAuth2Service,
              private router: Router,
              private translate: TranslateService,
              private _renderer: Renderer2,
              @Inject(DOCUMENT) private readonly _document: Document) {
    super();
    this.store.dispatch(new InitApplication());
    this.setTitle();
    this.listenCurrentModule();
    this.listenThemeChange();
  }

  useLanguage(language: LanguagesEnum): void {
    this.store.dispatch(new ChangeAppLanguage(language));
  }

  navigate(route: string): void {
    this.store.dispatch(new Navigate([route]));
  }

  private setTitle(): void {
    this._document.title = environment.appTitle;
  }

  private setTheme(): void {
    this._renderer.setAttribute(this._document.body, AppComponent._themeAttributeName, environment.theme);
  }

  private setFavicon(): void {
    const link = this._document.querySelector("link[rel*='icon']") || this._document.createElement("link");
    this._renderer.setAttribute(link, "type", "image/x-icon");
    this._renderer.setAttribute(link, "rel", "shortcut icon");
    this._renderer.setAttribute(link, "href", `./assets/themes/${environment.theme}/favicon.ico`);
  }

  private setImageToolbar(): void {
    this.logo = `assets/themes/${environment.theme}/toolbar-header-image.svg`;
  }

  private listenCurrentModule(): void {
    this.subscribe(this.urlStateObs
      .pipe(
        tap(urlState => {
          if (urlState) {
            const url = urlState.url;
            if (url.startsWith(urlSeparator + RoutesEnum.organizationalUnit)) {
              this.currentModule = RoutesEnum.organizationalUnit;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.access)) {
              this.currentModule = RoutesEnum.access;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.deposit)) {
              this.currentModule = RoutesEnum.deposit;
              return;
            }

            this.currentModule = null;
          }
        }),
      ),
    );
  }

  logout(): void {
    this.store.dispatch(new Logout());
  }

  getTitle(): string {
    return this._document.title;
  }

  themeChange(theme: ThemeEnum): void {
    this.store.dispatch(new ChangeAppTheme(theme));
  }

  private listenThemeChange(): void {
    this.subscribe(
      this.themeObs.pipe(
        tap((theme: ThemeEnum) => {
          this.setTheme();
          this.setFavicon();
          this.setImageToolbar();
        }),
      ),
    );
  }
}
