import {HttpEventType} from "@angular/common/http";
import {FileUploadStatusEnum} from "@app/features/deposit/enums/file-upload-status.enum";
import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {UploadFileStatus} from "@app/features/deposit/models/upload-file-status.model";
import {depositDataFileActionNameSpace} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {DepositDataFileState, DepositDataFileStateModel} from "@app/features/deposit/stores/data-file/deposit-data-file.state";
import {DepositAction, depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {Deposits} from "@app/generated-api";
import {ApiActionEnum} from "@app/shared/enums/api-action.enum";
import {PreIngestResourceApiEnum} from "@app/shared/enums/api.enum";
import {LanguageAction} from "@app/shared/stores/language/language.action";
import {LicenseAction} from "@app/shared/stores/license/license.action";
import {PreservationPolicyAction} from "@app/shared/stores/preservation-policy/preservation-policy.action";
import {SubmissionPolicyAction} from "@app/shared/stores/submission-policy/submission-policy.action";
import {Action, Actions, State, StateContext, Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {map} from "rxjs/operators";
import {
  ApiService,
  CompositionActionHelper,
  defaultResourceStateInitValue,
  ResourceApiEnum,
  ResourceNameSpace,
  ResourceState,
  ResourceStateModel,
  NotificationService,
} from "solidify-frontend";
import {LocalStateEnum} from "../../../shared/enums/local-state.enum";

export interface DepositStateModel extends ResourceStateModel<Deposits> {
  [LocalStateEnum.deposit_dataFile]: DepositDataFileStateModel;
  listDataFile: DepositDataFile[];
  isLoadingDataFile: boolean;
  uploadStatus: UploadFileStatus;
}

@State<DepositStateModel>({
  name: LocalStateEnum.deposit,
  defaults: {
    ...defaultResourceStateInitValue,
    [LocalStateEnum.deposit_dataFile]: null,
    listDataFile: [],
    isLoadingDataFile: false,
    uploadStatus: null,
  },
  children: [
    DepositDataFileState,
  ],
})
export class DepositState extends ResourceState<Deposits> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$);
  }

  defineUrlResource(): ResourceApiEnum {
    return PreIngestResourceApiEnum.deposits;
  }

  defineResourceNameSpace(): ResourceNameSpace {
    return depositActionNameSpace;
  }

  defineState(): LocalStateEnum {
    return LocalStateEnum.deposit;
  }

  @Action(DepositAction.LoadResource)
  loadResource(ctx: StateContext<DepositStateModel>, action: DepositAction.LoadResource): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    ctx.dispatch([
      new LanguageAction.GetAll(),
      new LicenseAction.GetAll(),
      new PreservationPolicyAction.GetAll(),
      new SubmissionPolicyAction.GetAll(),
      new DepositAction.LoadResourceSuccess(action),
    ]);
  }

  @Action(DepositAction.LoadResourceSuccess)
  loadResourceSuccess(ctx: StateContext<DepositStateModel>, action: DepositAction.LoadResourceSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(DepositAction.LoadResourceFail)
  loadResourceFail(ctx: StateContext<DepositStateModel>, action: DepositAction.LoadResourceFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(DepositAction.SendDataFile)
  sendDataFile(ctx: StateContext<DepositStateModel>, action: DepositAction.SendDataFile): Observable<DepositDataFile | string> {
    const formData = new FormData();
    formData.append("file", action.file, action.file.name);
    formData.append("category", "Package");
    formData.append("type", "Metadata");
    return this.apiService.upload<DepositDataFile>(`${PreIngestResourceApiEnum.deposits}/${action.parentId}/${ApiActionEnum.UL}`, formData)
      .pipe(
        map((event) => {
          switch (event.type) {
            case HttpEventType.UploadProgress:
              const progress = Math.round(100 * event.loaded / event.total);
              ctx.patchState({
                uploadStatus: {
                  file: action.file,
                  progressPercentage: progress,
                  status: FileUploadStatusEnum.inProgress,
                },
              });
              return;
            case HttpEventType.Response:
              ctx.dispatch(CompositionActionHelper.getAllSubResource(depositDataFileActionNameSpace, action.parentId));
              ctx.patchState({
                uploadStatus: {
                  file: action.file,
                  progressPercentage: null,
                  status: FileUploadStatusEnum.completed,
                  result: event.body,
                },
              });
              return event.body;
            default:
              return `Unhandled event: ${event.type}`;
          }
        }),
      );
  }
}
