import {ChangeDetectionStrategy, Component, OnInit, Output} from "@angular/core";
import {ActivatedRoute, NavigationEnd, Router} from "@angular/router";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {Breadcrumb} from "@app/shared/models/breadcrumb.model";
import {TranslateService} from "@ngx-translate/core";
import {BehaviorSubject, Observable} from "rxjs";
import {distinctUntilChanged} from "rxjs/internal/operators/distinctUntilChanged";
import {filter, map} from "rxjs/operators";
import {ObservableUtil, TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-breadcrumb",
  templateUrl: "./shared-breadcrumb.presentational.html",
  styleUrls: ["./shared-breadcrumb.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedBreadcrumbPresentational extends SharedAbstractPresentational implements OnInit {
  breadcrumbsObs: Observable<Breadcrumb[]>;

  private readonly _navigateBS: BehaviorSubject<string | undefined> = new BehaviorSubject<string | undefined>(undefined);

  @Output("navigateChange")
  readonly navigateObs: Observable<string | undefined> = ObservableUtil.asObservable(this._navigateBS);

  private readonly _BREADCRUMB_KEY: string = "breadcrumb";
  private readonly _BREADCRUMB_TRANSLATE_HOME: string = TRANSLATE("breadcrumb.home");

  constructor(private translate: TranslateService,
              private router: Router,
              private activatedRoute: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.breadcrumbsObs = this.router.events
      .pipe(
        filter(event => event instanceof NavigationEnd),
        distinctUntilChanged(),
        map(event => this.buildBreadCrumb(this.activatedRoute.root)),
      );
  }

  buildBreadCrumb(route: ActivatedRoute,
                  url: string = "",
                  breadcrumbs: Breadcrumb[] = []): Breadcrumb[] {
    const label = this.getLabel(route);
    const path = this.getPath(route);

    const fullUrl = `${url}${path}/`;

    const breadcrumb: Breadcrumb = {
      label,
      url: fullUrl,
    } as Breadcrumb;

    const newBreadcrumbs: Breadcrumb[] = [...breadcrumbs, breadcrumb];
    if (route.firstChild) {
      return this.buildBreadCrumb(route.firstChild, fullUrl, newBreadcrumbs);
    }
    return newBreadcrumbs;
  }

  private getLabel(route: ActivatedRoute): string {
    if (route.routeConfig) {
      const routeConfig = route.routeConfig;
      if (routeConfig.data == null || routeConfig.data[this._BREADCRUMB_KEY] == null) {
        throw Error(`Please define the data '${this._BREADCRUMB_KEY}', for the following route ${route.toString()}`);
      }
      return route.routeConfig.data[this._BREADCRUMB_KEY];
    }

    return this._BREADCRUMB_TRANSLATE_HOME;
  }

  private getPath(route: ActivatedRoute): string {
    if (route.routeConfig) {
      let path = route.routeConfig.path;
      path = this.insertParametersInPath(route, path);
      return path;
    }
    return "";
  }

  private insertParametersInPath(route: ActivatedRoute, path: string): string {
    const params = route.snapshot.params;
    // tslint:disable-next-line:forin
    for (const key in params) {
      path = path.replace(`:${key}`, params[key]);
    }
    return path;
  }

  navigate(url: string): void {
    this._navigateBS.next(url);
  }
}
