import {ChangeDetectionStrategy, Component, Input, OnChanges, Output, SimpleChanges, ViewChild, ViewEncapsulation} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {SharedPaginatorPresentational} from "@app/shared/components/presentationals/shared-paginator/shared-paginator.presentational";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {DataTableColumns} from "@app/shared/models/data-table-columns.model";
import _ from "lodash";
import * as moment from "moment";
import {LazyLoadEvent} from "primeng/api";
import {BehaviorSubject, Observable} from "rxjs";
import {
  isEmptyString,
  MapUtil,
  ObjectUtil,
  ObservableUtil,
  OrderEnum,
  Paging,
  QueryParameters,
  QueryParametersUtil,
  Sort,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-data-table",
  templateUrl: "./shared-data-table.presentational.html",
  styleUrls: ["./shared-data-table.presentational.scss"],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedDataTablePresentational<T> extends SharedAbstractPresentational implements OnChanges {
  @Input()
  datas: T[];

  @Input()
  queryParameters: QueryParameters;

  @Input()
  columns: DataTableColumns[];

  private readonly _selectBS: BehaviorSubject<T | undefined> = new BehaviorSubject<T | undefined>(undefined);
  private readonly _queryParametersBS: BehaviorSubject<QueryParameters | undefined> = new BehaviorSubject<QueryParameters | undefined>(undefined);

  @Output("selectChange")
  readonly selectObs: Observable<T | undefined> = ObservableUtil.asObservable(this._selectBS);

  @Output("queryParametersChange")
  readonly queryParametersObs: Observable<QueryParameters | undefined> = ObservableUtil.asObservable(this._queryParametersBS);

  @ViewChild("paginator")
  paginator: SharedPaginatorPresentational;

  isDatasPresent: boolean;

  fieldTypeString: FieldTypeEnum = FieldTypeEnum.string;

  private readonly _SEPARATOR: string = ".";
  private readonly _FORMAT_TIME_ONLY: string = "LTS";
  private readonly _FORMAT_DATE_ONLY: string = "L";

  constructor() {
    super();
  }

  ngOnChanges(changes: SimpleChanges): void {
    super.ngOnChanges(changes);
    this.computeIsDatasPresent();
  }

  private computeIsDatasPresent(): void {
    this.isDatasPresent = this.datas.length > 0;
  }

  showDetail(row: T): void {
    this._selectBS.next(row);
  }

  getCellData(row: T, column: DataTableColumns): any {
    const data = this.getData(row, column);
    return this.applyTypeToData(data, column.type);
  }

  private getData(row: T, column: DataTableColumns): any {
    const field = column.field;
    if (field.includes(this._SEPARATOR)) {
      return this.getNestedData(column, row);
    }
    return row[field];
  }

  private getNestedData(column: DataTableColumns, row: T): void {
    const nestedProperties: string[] = column.field.split(this._SEPARATOR);
    return row[_.last(nestedProperties)];
  }

  private applyTypeToData(data: any, type: FieldTypeEnum): any {
    if (type === FieldTypeEnum.date) {
      return moment(data).format(this._FORMAT_DATE_ONLY);
    }
    if (type === FieldTypeEnum.datetime) {
      return moment(data).format(this._FORMAT_DATE_ONLY) + " " + moment(data).format(this._FORMAT_TIME_ONLY);
    }
    return data;
  }

  backendSort($event: LazyLoadEvent): void {
    if ($event.sortField === undefined) {
      return;
    }
    const sort = this.adaptLazyLoadEventToSortModel($event);
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    queryParameters.sort = sort;
    this._queryParametersBS.next(queryParameters);
  }

  private adaptLazyLoadEventToSortModel($event: LazyLoadEvent): Sort {
    return {
      field: $event.sortField,
      order: $event.sortOrder === -1 ? OrderEnum.descending : OrderEnum.ascending,
    } as Sort;
  }

  pageChange($event: Paging): void {
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    queryParameters.paging = $event;
    this._queryParametersBS.next(queryParameters);
  }

  onValueChange(col: DataTableColumns, value: string): void {
    let queryParameters = ObjectUtil.clone(this.queryParameters);
    if (isEmptyString(value)) {
      queryParameters.search.searchItems.delete(col.field);
    } else {
      MapUtil.addOrUpdate(queryParameters.search.searchItems, col.field, value);
    }
    queryParameters = QueryParametersUtil.resetToFirstPage(queryParameters);
    this._queryParametersBS.next(queryParameters);
  }

  getValue(field: string): string {
    const value = this.queryParameters.search.searchItems.get(field);
    if (value === null || value === undefined || value === "") {
      return null;
    }
    return value;
  }
}
