import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {Deposits, OrganizationalUnits} from "@app/generated-api";
import {AccessResourceApiEnum, AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {LocalStateModel} from "@app/shared/models/localStateModel";
import {OrgUnitAction, orgUnitActionNameSpace} from "@app/shared/stores/organizational-unit/organizational-unit.action";
import {orgUnitSubmissionPolicyActionNameSpace} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.action";
import {
  OrganizationalUnitSubmissionPolicyState,
  OrganizationalUnitSubmissionPolicyStateModel,
} from "@app/shared/stores/organizational-unit/submission-policy/organizational-unit-submission-policy.state";
import {PreservationPolicyAction} from "@app/shared/stores/preservation-policy/preservation-policy.action";
import {SubmissionPolicyAction} from "@app/shared/stores/submission-policy/submission-policy.action";
import {Action, Actions, ofActionCompleted, State, StateContext, Store} from "@ngxs/store";

import _ from "lodash";
import {Observable} from "rxjs";
import {catchError} from "rxjs/internal/operators/catchError";
import {tap} from "rxjs/internal/operators/tap";
import {
  ApiService,
  CollectionTyped,
  defaultResourceStateInitValue,
  ResourceApiEnum,
  ResourceNameSpace,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  Relation2TiersActionHelper,
  OverrideDefaultAction,
  NotificationService,
} from "solidify-frontend";
import {LocalStateEnum} from "../../enums/local-state.enum";

export interface OrganizationalUnitStateModel extends ResourceStateModel<OrganizationalUnits> {
  [LocalStateEnum.organizationalUnit_submissionPolicy]?: OrganizationalUnitSubmissionPolicyStateModel;
}

@State<OrganizationalUnitStateModel>({
  name: LocalStateEnum.organizationalUnit,
  defaults: {
    ...defaultResourceStateInitValue,
  },
  children: [
    OrganizationalUnitSubmissionPolicyState,
  ],
})
export class OrganizationalUnitState extends ResourceState<OrganizationalUnits> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$);
  }

  defineUrlResource(): ResourceApiEnum {
    const isLoggedId = this.store.selectSnapshot((state: LocalStateModel) => state[LocalStateEnum.application].isLoggedIn);
    if (isLoggedId) {
      return AdminResourceApiEnum.organizationalUnits;
    }
    return AccessResourceApiEnum.organizationalUnits;
  }

  defineResourceNameSpace(): ResourceNameSpace {
    return orgUnitActionNameSpace;
  }

  defineState(): LocalStateEnum {
    return LocalStateEnum.organizationalUnit;
  }

  @Action(OrgUnitAction.LoadResource)
  loadResource(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.LoadResource): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    ctx.dispatch([
      new PreservationPolicyAction.GetAll(),
      new SubmissionPolicyAction.GetAll(),
      new OrgUnitAction.LoadResourceSuccess(action),
    ]);
  }

  @Action(OrgUnitAction.LoadResourceSuccess)
  loadResourceSuccess(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.LoadResourceSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(OrgUnitAction.LoadResourceFail)
  loadResourceFail(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.LoadResourceFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(OrgUnitAction.GetAllFromAccess)
  getAllAccess(ctx: StateContext<DepositStateModel>, action: OrgUnitAction.GetAllFromAccess): Observable<CollectionTyped<Deposits>> {
    // TOFIX Find a better way to manage get all Admin / get all Access (different store ?)
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      queryParameters: StoreUtil.getQueryParametersToApply(action.queryParameters, ctx),
    });
    return this.apiService.get<Deposits>(AccessResourceApiEnum.organizationalUnits, ctx.getState().queryParameters)
      .pipe(
        tap((collection: CollectionTyped<Deposits>) => {
          ctx.dispatch(new OrgUnitAction.GetAllSuccess(action, collection));
        }),
        catchError(error => {
          ctx.dispatch(new OrgUnitAction.GetAllFail(action));
          throw error;
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(OrgUnitAction.Create)
  create(ctx: StateContext<OrganizationalUnitStateModel>, action: OrgUnitAction.Create): Observable<OrganizationalUnits> {
    return super.internalCreate(ctx, action)
      .pipe(
        tap(model => {
          const newSubmissionPolicies = action.modelFormControlEvent[LocalModelAttributeEnum.submissionPolicies];

          StoreUtil.dispatchActionAndWaitForSubActionsCompletion(
            ctx,
            [
              {
                action: Relation2TiersActionHelper.updateSubResource(orgUnitSubmissionPolicyActionNameSpace, model.resId, [], newSubmissionPolicies),
                subActionCompletions: [
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateSuccess)),
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateFail)),
                ],
              },
            ],
          ).subscribe(success => {
            if (success) {
              ctx.dispatch(new orgUnitActionNameSpace.CreateSuccess());
            } else {
              ctx.dispatch(new orgUnitActionNameSpace.CreateFail());
            }
          });
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(OrgUnitAction.Update)
  update(ctx: StateContext<OrganizationalUnitStateModel>, action: OrgUnitAction.Update): Observable<OrganizationalUnits> {
    return super.internalUpdate(ctx, action)
      .pipe(
        tap(model => {
          const oldSubmissionPolicies = _.map(ctx.getState()[LocalStateEnum.organizationalUnit_submissionPolicy].selected, LocalModelAttributeEnum.resId);
          const newSubmissionPolicies = action.modelFormControlEvent.model[LocalModelAttributeEnum.submissionPolicies];

          StoreUtil.dispatchActionAndWaitForSubActionsCompletion(
            ctx,
            [
              {
                action: Relation2TiersActionHelper.updateSubResource(orgUnitSubmissionPolicyActionNameSpace, action.modelFormControlEvent.model.resId, oldSubmissionPolicies, newSubmissionPolicies),
                subActionCompletions: [
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateSuccess)),
                  this.actions$.pipe(ofActionCompleted(orgUnitSubmissionPolicyActionNameSpace.UpdateFail)),
                ],
              },
            ],
          ).subscribe(success => {
            if (success) {
              ctx.dispatch(new orgUnitActionNameSpace.UpdateSuccess());
            } else {
              ctx.dispatch(new orgUnitActionNameSpace.UpdateFail());
            }
          });
        }),
      );
  }
}
